from .appcache import appcache
from .builder import build
from .collect import collectjson
from .compilers import optimize, scss, mustache
from .init import init
from .setversion import setversion
from .icons import icons
from .phonegap import phonegap

__all__ = (
    "appcache",
    "build",
    "collectjson",
    "optimize",
    "scss",
    "mustache",
    "init",
    "setversion",
    "icons",
    "phonegap",
)



from http.server import HTTPServer, SimpleHTTPRequestHandler
from html_json_forms import parse_json_form
from urllib.parse import parse_qs
import json
import time
import random


class RequestHandler(SimpleHTTPRequestHandler):
    def echo(self, **update):
        length = int(self.headers.get('content-length'))
        qs = self.rfile.read(length).decode('utf-8')
        data = parse_json_form({
            key: val[0]
            for key, val in parse_qs(qs).items()
        })
        for value in data.values():
            if isinstance(value, list):
                for i, val in enumerate(value):
                    val['@index'] = i
        data.update(**update)
        self.send_response(200)
        self.send_header("Content-Type", "application/json")
        self.end_headers()
        # FIXME: This is to avoid a race condition in wq/outbox
        time.sleep(random.random())
        self.wfile.write(json.dumps(data).encode('utf-8'))

    def do_POST(self):
        pk = ''.join([chr(ord('a') + random.randint(0, 25)) for o in range(3)])
        self.echo(id=pk)

    def do_PUT(self):
        pk = self.path.split('/')[-1].split('.')[0]
        self.echo(id=pk)


def run():
    address = ("", 8080)
    httpd = HTTPServer(address, RequestHandler)
    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        httpd.server_close()

if __name__ == "__main__":
    run()



import click
from wq.core import wq
from os.path import exists, join, dirname
from os import symlink, mkdir


@wq.command()
@click.option(
    '--js', default='js', type=click.Path(),
    help='Path to JS folder (default js/)'
)
@click.option(
    '--css', default='css', type=click.Path(),
    help='Path to CSS folder (default css/)'
)
@click.option(
    '--scss', default='scss', type=click.Path(),
    help='Path to SCSS/SASS folder (default scss/)'
)
def init(**conf):
    """
    Link js, css, and scss to wq.app libs.  This makes it possible to leverage
    wq.app's assets via short relative paths without vendoring the entire
    wq.app codebase in your project.  A "lib/" folder will be configured for
    each of the three asset types.

    While symbolic links are supported on all modern operating systems, some
    (i.e. Windows) may require administrative access.  After running wq init as
    an administrator, you should be able to continue as a standard user.

    Note: It's best to configure VCS to completely ignore the lib/ entries -
    since they may be different on each computer, and they can be created
    automatically as needed.
    """

    def maybe_symlink(source, dest):
        try:
            symlink(source, dest)
        except OSError as e:
            raise click.ClickException(
                "%s.\n\nSee wq init --help for more info" % (e)
            )

    for name in ('js', 'css', 'scss'):
        basedir = conf[name]
        if not exists(basedir):
            continue

        # Project lib directory (e.g. myapp/js/lib)
        projpath = join(basedir, 'lib')
        # wq source directory (e.g. wq.app/js)
        wqpath = join(dirname(dirname(__file__)), name)

        if not exists(projpath):
            # e.g myapp/js/lib -> wq.app/js
            if name == "scss":
                # scss/lib should always be a folder
                mkdir(projpath)
            else:
                maybe_symlink(wqpath, projpath)
        if exists(join(wqpath, 'wq')) and not exists(join(projpath, 'wq')):
            # e.g myapp/js/lib/wq -> wq.app/js/wq
            maybe_symlink(join(wqpath, 'wq'), join(projpath, 'wq'))
        if name == "scss" and not exists(join(projpath, 'compass')):
            # myapp/scss/lib/compass -> compass_stylesheets/stylesheets/compass
            import pkg_resources
            compass = pkg_resources.resource_filename(
                'compass_stylesheets',
                join('stylesheets', 'compass'),
            )
            maybe_symlink(compass, join(projpath, 'compass'))



from wq.core import wq
import click

import os
import json

import scss as pyScss
import logging
import pystache

from .collect import readfiles
import requirejs
from babeljs import transformer as babeljs


@wq.command()
@wq.pass_config
def optimize(config):
    """
    Use r.js to optimize JS and CSS assets.  This command requires an
    "optimize" section in your configuration file, which will be passed as-is
    to r.js for compilation.  See http://requirejs.org/docs/optimization.html
    for available configuration options.
    """
    conf = config.get('optimize', None)
    if not conf:
        raise click.UsageError(
            "optimize section not found in %s" % config.filename
        )

    # Defer to r.js for actual processing
    click.echo("Optimizing with r.js...")
    try:
        requirejs.optimize(conf)
    except requirejs.RJSException as e:
        raise click.ClickException(e.args[0])

    click.echo("Optimization complete")


@wq.command()
@wq.pass_config
def babel(config):
    """
    Use babel.js to compile ES6/2015+.  Generates ES5-compatible JavaScript for
    older browsers.  Note that wq babel is run after wq optimize, on the
    compiled modules created by r.js.  Support for running babel at other
    stages of the build process may be added in a future version of wq.app.
    """
    rconf = config.get('optimize', None)
    if not rconf:
        raise click.UsageError(
            "optimize section not found in %s" % config.filename
        )

    babel = config.get('babel', {})
    files = []
    if 'modules' in rconf and 'dir' in rconf:
        base_url = rconf.get('baseUrl', '.')
        for module in rconf['modules']:
            path = module['name']
            if path in rconf.get('paths', {}):
                path = rconf['paths'][path]
            path = os.path.join(rconf['dir'], base_url, path)
            files.append(path + '.js')

    for filename in files:
        label = os.path.normpath(filename)
        try:
            with open(filename) as f:
                content = f.read()
        except OSError:
            raise click.ClickException(
                "Error loading %s - run wq optimize first?" % label
            )
        try:
            print("Transforming %s with Babel..." % label)
            output = babeljs.transform_string(content, **babel)
        except babeljs.TransformError as e:
            raise click.ClickException(e.args[0])
        with open(filename, 'w') as f:
            f.write(output)


@wq.command()
@click.option(
    '--indir', type=click.Path(exists=True), default="scss",
    help="Path to SCSS/SASS files"
)
@click.option(
    '--outdir', type=click.Path(exists=True), default="css",
    help="Path to CSS files"
)
def scss(**conf):
    """
    Render all SCSS/SASS files into CSS.  The input directory will be searched
    for *.scss files, which will be compiled to corresponding *.css files in
    the output directory.
    """
    compiler = pyScss.Scss(scss_opts={'compress': 0})
    logging.getLogger("scss").addHandler(logging.StreamHandler())

    def compile(path, source):
        css = compiler.compile(source)
        outfile = open(path, 'w')
        outfile.write(css)
        outfile.close()

    files = readfiles(conf['indir'], "scss")
    pyScss.config.LOAD_PATHS = [
        conf['indir'],
        os.path.join(conf['indir'], 'lib'),

        # FIXME: Why aren't these paths automatically picked up on Windows?
        os.path.join(conf['indir'], 'lib', 'compass'),
        os.path.join(conf['indir'], 'lib', 'compass', 'css3'),
    ]
    for name, source in files.items():
        if isinstance(source, dict):
            continue
        path = "%s/%s.css" % (conf['outdir'], name)
        compile(path, source)
        click.echo("%s compiled from %s/%s.scss" % (path, conf['indir'], name))


@wq.command()
@click.option('--template', help="Path to template")
@click.option('--partials', help="Path to partials")
@click.option('--context', help="Path to context (JSON or YAML)")
@click.option(
    '--output', type=click.Path(), default="output.html",
    help="Output filename"
)
def mustache(**conf):
    """
    Render a mustache template into static HTML.  The template context can be
    provided via a nexted object in wq.yml, or by pointing to a folder
    containing JSON or YAML files.  Similarly, the partials can be defined as a
    nested object in wq.yml or by a folder path.

    Example YAML configuration:

    \b
    mustache:
        template: "<html><body>{{>header}}{{>footer}}</body></html>"
        partials:
            header: "<h3>{{title}}</h3>"
            footer: "<a href='mailto:{{email}}'>{{email}}</a>"
        context:
            title: "Example"
            email: "email@example.com"
        output: index.html

    Example command line configuration:

    wq mustache --template tmpl.html --partials partials/ --context conf/
    """
    template = conf['template']
    if template is None:
        return
    if os.path.exists(template) or template.endswith('.html'):
        try:
            template = open(template).read()
        except IOError as e:
            raise click.FileError(template, hint=str(e))

    context = conf["context"] or {}
    if not isinstance(context, dict):
        if context.startswith('{'):
            context = json.loads(context)
        else:
            path = context
            context = readfiles(path, "yaml", "yml")
            context.update(**readfiles(path, "json"))

    partials = conf['partials'] or {}
    if not isinstance(partials, dict):
        partials = readfiles(partials, "html")

    click.echo("Generating %s from %s" % (conf['output'], conf['template']))
    renderer = pystache.Renderer(partials=partials)
    html = renderer.render(template, context)
    f = open(conf['output'], 'w')
    f.write(html)
    f.close()



from wq.core import wq
import click

import os
from PIL import Image, ImageOps

SIZES = {
    'web': {
        180: 'apple-touch-icon',
        192: 'manifest-icon',
        512: 'manifest-splash-icon',
    },

    'android': {
        36: 'ldpi',
        48: 'mdpi',
        72: 'hdpi',
        96: 'xhdpi',
        144: 'xxhdpi',
        192: 'xxxhdpi',
    },

    'android-splash': {
        '100.9': 'ldpi.9',
        '160.9': 'mdpi.9',
        '240.9': 'hdpi.9',
        '360.9': 'xhdpi.9',
        '480.9': 'xxhdpi.9',
        '640.9': 'xxxhdpi.9',
    },

    'ios': {
        29: 'icon-small',
        40: 'icon-40',
        58: 'icon-small@2x',
        60: 'icon-60',
        76: 'icon-76',
        80: 'icon-40@2x',
        87: 'icon-small@3x',
        120: 'icon-40@3x',
        152: 'icon-76@2x',
        167: 'icon-83.5@2x',
        180: 'icon-60@3x',

        # iOS 6.1
        50: 'icon-50',
        57: 'icon',
        72: 'icon-72',
        100: 'icon-50@2x',
        114: 'icon@2x',
        144: 'icon-72@2x',
    },

    'ios-splash': {
        '320x480': 'Default',
        '640x960': 'Default@2x',
        '640x1136': 'Default-568h@2x',
        '750x1334': 'Default-667h@2x',
        '768x1024': 'Default-Portrait~ipad',
        '1024x768': 'Default-Landscape~ipad',
        '1242x2208': 'Default-Portrait-736h@3x',
        '1536x2048': 'Default-Portrait@2x~ipad',
        '2048x1536': 'Default-Landscape@2x~ipad',
        '2208x1242': 'Default-Landscape-736h@3x',
        2732: 'Default@2x~universal~anyany',
    },

    'windows': {
        30: 'Square30x30Logo',
        44: 'Square44x44Logo',
        50: 'StoreLogo',
        70: 'Square70x70Logo',
        71: 'Square71x71Logo',
        106: 'Square44x44Logo.scale-240',
        120: 'StoreLogo.scale-240',
        150: 'Square150x150Logo',
        170: 'Square71x71Logo.scale-240',
        310: 'Square310x310Logo',
        '310x150': 'Wide310x150Logo',
        360: 'Square150x150Logo.scale-240',
        '744x360': 'Wide310x150Logo.scale-240',
    },

    'windows-splash': {
        '620x300': 'SplashScreen',
        '1152x1920': 'SplashScreenPhone.scale-240',
    }
}

PLATFORMS = set()
ALIASES = {}
for platform, aliases in list(SIZES.items()):
    PLATFORMS.add(platform)
    for size, alias in aliases.items():
        SIZES[alias] = {size: alias}
        ALIASES.setdefault(size, {})
        ALIASES[size][platform] = alias


@wq.command()
@click.argument('source', type=click.Path(exists=True))
@click.option(
    '--size', '-s', multiple=True, help="Pixel size (or name of platform)",
)
@click.option(
    '--filename', '-n', default="icon-{size}.png",
    help="Filename template for output files"
)
@click.option(
    '--outdir', '-d', type=click.Path(), help="Output Directory"
)
def icons(**conf):
    """
    Generate resized icons from source image.  If no size is specified,
    generates all of the recommended icon sizes for web, Android, iOS, and
    Windows apps.
    """
    if not conf['size']:
        conf['size'] = SIZES.keys()
    elif all([isinstance(c, str) and len(c) == 1 for c in conf['size']]):
        conf['size'] = ("".join(conf['size']),)

    platform = None
    sizes = set()
    for size in conf['size']:
        if size in SIZES:
            sizes.update(SIZES[size].keys())
            if size in PLATFORMS and not platform:
                # Use first specified platform as default for icon aliases
                platform = size
        elif isinstance(size, int):
            sizes.add(size)
        elif size.isdigit():
            sizes.add(int(size))
        elif len(size.split('x')) == 2:
            sizes.add(size)
        elif size.endswith('.9'):
            sizes.add(size)

    if not sizes:
        click.echo("Size not recognized: %s" % str(conf['size']))
        return

    def sortkey(size):
        if isinstance(size, int):
            return size
        elif size.endswith('.9'):
            return float(size)
        else:
            return max(int(s) for s in size.split('x'))

    sort_sizes = [str(size) for size in sorted(sizes, key=sortkey)]
    if platform and len(conf['size']) == 1:
        platform_note = " for %s" % platform
    else:
        platform_note = ""
    click.echo(
        "Generating icons%s: %s" % (platform_note, ", ".join(sort_sizes))
    )

    if conf['outdir'] and not os.path.exists(conf['outdir']):
        os.mkdir(conf['outdir'])

    img = Image.open(conf['source'])
    for size in sizes:
        nine_patch = False
        if isinstance(size, int):
            width = height = minsize = size
        elif size.endswith('.9'):
            width = height = minsize = int(size.replace('.9', ''))
            nine_patch = True
        else:
            width, height = (int(s) for s in size.split('x'))
            minsize = min(width, height)
        minsize = min(minsize, max(img.width, img.height))

        icon = img.copy()
        icon.thumbnail((minsize, minsize), Image.ANTIALIAS)
        if icon.width < width or icon.height < height:
            lrpad = (width - icon.width)
            left = lrpad // 2
            right = lrpad - left
            tbpad = (height - icon.height)
            top = tbpad // 2
            bottom = tbpad - top
            fill = img.load()[0, 0]
            icon = ImageOps.expand(icon, (left, top, right, bottom), fill)

        aliases = ALIASES.get(size, {})
        if len(aliases.keys()) == 1:
            alias = list(aliases.values())[0]
        elif platform and platform in aliases:
            alias = aliases[platform]
        else:
            alias = 'icon-%s' % size

        if nine_patch:
            icon = ImageOps.expand(icon, 1, 0)
            data = icon.load()
            if icon.mode == 'LA':
                data[0, 1] = data[1, 0] = (0, 255)
                data[0, height] = data[width, 0] = (0, 255)
            else:
                data[0, 1] = data[1, 0] = (0, 0, 0, 255)
                data[0, height] = data[width, 0] = (0, 0, 0, 255)

        name = conf['filename'].format(
            size=size,
            alias=alias,
        )
        if conf['outdir']:
            name = os.path.join(conf['outdir'], name)
        icon.save(name, 'PNG')



import os
import json
import yaml
from wq.core import wq
import click
from collections import OrderedDict


NEST = {
    'json': json,
    'yaml': yaml
}


def readfiles(basedir, ftype=None, fext=None):
    obj = OrderedDict()
    if fext is None:
        fext = ftype

    for path, dirs, files in sorted(os.walk(basedir)):
        if '.svn' in dirs:
            dirs.remove('.svn')
        o = obj
        if path == basedir:
            path = ""
        else:
            path = path[len(basedir) + 1:]
            apath = path.split(os.sep)
            for subdir in apath:
                o = o[subdir]
            path = os.sep.join(apath) + os.sep

        for filename in sorted(files):
            name, ext = os.path.splitext(filename)
            if ftype and ext != '.' + fext:
                continue

            fpath = path + name
            data = open(basedir + os.sep + fpath + ext)
            if ftype in NEST:
                try:
                    o[name] = NEST[ftype].load(data)
                except ValueError as e:
                    raise click.ClickException(
                        "Could not parse %s; %s" % (fpath + ext, e)
                    )
            else:
                o[name] = data.read()

        for name in sorted(dirs):
            o[name] = OrderedDict()

    return obj


@wq.command()
@click.option(
    '--type', default='json', help="Source file type (e.g. json, yaml)"
)
@click.option('--extension', help="Source file extension (e.g. json, yml)")
@click.option('--output', default='output.json', help="Destination JSON file")
@click.option('--indent', default=4, help="JSON Indentation")
@click.option('--jsonp', help="Wrap as JSONP")
@click.argument('paths', type=click.Path(exists=True), nargs=-1)
@wq.pass_config
def collectjson(config, **conf):
    """
    Load directory files into a JSON object.  The filenames will become the
    keys and the contents will become the values.  The most common use for this
    tool is to collect a group of Mustache templates into an AMD module, e.g.:

    wq collectjson --type html --output templates.js --jsonp define templates/

    This tool can also be used to collect existing JSON or YAML configuration
    files into a single object.  In that case, the file contents will be
    embeded as full JSON objects instead of strings.

    wq collectjson --type json --output config.json config/
    """

    if not conf['extension']:
        conf['extension'] = conf['type']

    if not conf['paths']:
        if isinstance(config.get('collectjson', None), dict):
            # Ensure wq.yml paths are used if not passed via argument
            conf['paths'] = config['collectjson'].get('paths', None)

        if not conf['paths']:
            conf['paths'] = ['.']

    obj = OrderedDict()
    for d in conf['paths']:
        obj.update(readfiles(d, conf['type'], conf['extension']))

    outfile = open(conf['output'], 'w')

    opts = {}
    if conf['indent']:
        opts['indent'] = conf['indent']

    if conf['jsonp']:
        txt = json.dumps(obj, **opts)
        txt = '%s(%s);' % (conf['jsonp'], txt)
        outfile.write(txt)
    else:
        json.dump(obj, outfile, **opts)

    click.echo('%s: %s objects collected from %s' % (
        conf['output'], len(obj), ', '.join(conf['paths'])
    ))

    outfile.close()



import requests
from wq.core import wq
import click
import os
import yaml
import json
import shutil
import pystache
from .icons import icons, SIZES


@wq.command()
@click.option(
    '--source', type=click.Path(), help="Directory containing app assets"
)
@click.option('--icon', type=click.Path(), help="Source image for icons")
@click.option(
    '--splash', type=click.Path(),
    help="Source image for splash screen (defaults to icon)",
)
@click.option('--config-xml', type=click.Path(), help="config.xml template")
@click.option('--index-html', type=click.Path(), help="index.html template")
@click.option(
    '--pgb-directory', type=click.Path(), default=".wq-pgb",
    help="Temporary working directory (default .wq-pgb)"
)
@click.option(
    '--pgb-api', default="https://build.phonegap.com/api/v1/",
    help="PhoneGap Build API URL",
)
@click.option(
    '--pgb-token-api', default="https://build.phonegap.com/token",
    help="PhoneGap Build Token API URL",
)
@click.argument('version')
@wq.pass_config
@click.pass_context
def phonegap(ctx, config, version, **conf):
    """
    Upload application to PhoneGap Build.  Specifically,

    \b
    1. Create a working directory, if not present.
    2. Copy asset directory to working directory.
    3. (Optionally) Generate config.xml and index.html from mustache templates.
    4. Generate zip file.
    5. Request an authentication token from PhoneGap Build, if not present.
    6. Upload the zip file to PhoneGap build.
    7. Save the returned app ID for future builds.
    \b
    (Inspired by, but not dependent on, the `phonegap remote` api)
    """

    if conf['source']:
        source = conf['source']
    else:
        source = config.get('optimize', {}).get('dir', None)
        if not source:
            raise click.UsageError(
                "Set --source or define optimize section in wq.yml"
            )

    directory = conf['pgb_directory']

    if not os.path.exists(directory):
        choice = ''
        while choice.lower() not in ('y', 'n', 'd'):
            choice = click.prompt('Configure PhoneGap Build? [y/n/d/?]')
            if choice.lower() == '?':
                click.echo(
                    '  y - continue\n'
                    '  n - cancel for now\n'
                    '  d - cancel and disable\n'
                    '  ? - show help'
                )
        if choice == 'n':
            return
        else:
            os.mkdir(directory)
            if choice == 'd':
                set_pgb_config(directory, 'disable', True)

    if get_pgb_config(directory, 'disable'):
        click.echo(
            "PhoneGap Build disabled.  Remove {directory} to enable"
            .format(directory=directory)
        )
        return

    filename = create_zipfile(
        directory=directory,
        source=source,
        version=version,
        context=ctx,
        icon=conf['icon'],
        splash=conf['splash'] or conf['icon'],
        config_xml=conf['config_xml'],
        index_html=conf['index_html'],
    )

    token = get_token(directory, conf['pgb_token_api'])
    if not token:
        click.echo("No token found, stopping.", err=True)
        return

    upload_zipfile(directory, filename, token, conf['pgb_api'])


def create_zipfile(directory, source, version, context,
                   icon=None, splash=None, config_xml=None, index_html=None):
    folder = os.path.join(directory, 'build')
    if os.path.exists(folder):
        shutil.rmtree(folder)
    shutil.copytree(source, folder)

    template_context = {
        'version': version,
    }

    if icon or splash:
        icon_dir = 'icons'
        filename = '{alias}.png'
        platforms = tuple()
        if icon:
            platforms += ('android', 'ios', 'windows')
        if splash:
            platforms += ('android-splash', 'ios-splash', 'windows-splash')

        icon_path = os.path.join(directory, 'build', icon_dir)
        os.mkdir(icon_path)
        click.open_file(os.path.join(icon_path, '.pgbomit'), 'w').write('')

        def get_width(size):
            if isinstance(size, int):
                return size
            elif size.endswith('.9'):
                return int(size.replace('.9', ''))
            else:
                return int(size.split('x')[0])

        def get_height(size):
            if isinstance(size, int):
                return size
            elif size.endswith('.9'):
                return int(size.replace('.9', ''))
            else:
                return int(size.split('x')[1])

        for platform in platforms:
            context.invoke(
                icons,
                source=splash if platform.endswith('splash') else icon,
                outdir=icon_path,
                filename=filename,
                size=[platform],
            )
            sizes = [(
                get_width(size),
                get_height(size),
                size,
                alias,
            ) for size, alias in SIZES[platform].items()]
            template_context[platform] = {
                'icons': [{
                    'width': width,
                    'height': height,
                    'alias': alias,
                    'density': (
                        alias.replace('.9', '') if 'dpi' in alias else None
                    ),
                    'filename': os.path.join(
                        icon_dir,
                        filename.format(alias=alias)
                    )
                } for width, height, size, alias in sorted(sizes)]
            }

    if config_xml:
        xml = click.open_file(config_xml).read()
        xml = pystache.render(xml, template_context)
        click.open_file(os.path.join(folder, 'config.xml'), 'w').write(xml)

    if index_html:
        html = click.open_file(index_html).read()
        html = pystache.render(html, template_context)
        click.open_file(os.path.join(folder, 'index.html'), 'w').write(html)

    return shutil.make_archive(folder, 'zip', folder)


def upload_zipfile(directory, filename, token, pgb_api):
    pgb_url = pgb_api + "{path}?auth_token={token}"
    app_id = get_pgb_config(directory, 'app_id')
    if app_id:
        url = pgb_url.format(
            path='apps/{app_id}'.format(app_id=app_id),
            token=token,
        )
        method = 'put'
    else:
        url = pgb_url.format(
            path='apps',
            token=token,
        )
        method = 'post'

    response = requests.request(
        method,
        url,
        data={
            'data': json.dumps({'create_method': 'file'}),
        },
        files={
            'file': click.open_file(filename, 'rb'),
        }
    )
    if check_error(response):
        return
    result = response.json()
    if not app_id:
        set_pgb_config(directory, 'app_id', result['id'])
    click.echo("URL: {share_url}".format(share_url=result['share_url']))
    error = result.get('error', None)
    if error:
        click.echo("PGB Warning: {error}".format(error=error))


def get_pgb_config(directory, name=None):
    conf_path = os.path.join(directory, 'wq-pgb.yml')
    conf = {}
    if os.path.exists(conf_path):
        conf = yaml.load(click.open_file(conf_path))
        if not isinstance(conf, dict):
            conf = {}

    if name:
        return conf.get(name, None)
    else:
        return conf


def set_pgb_config(directory, name, value):
    conf = get_pgb_config(directory)
    conf_path = os.path.join(directory, 'wq-pgb.yml')
    conf[name] = value
    yaml.dump(conf, click.open_file(conf_path, 'w'), default_flow_style=False)


def check_error(response):
    if response.status_code in (200, 201):
        return False
    result = response.json()
    if 'error' in result:
        error = result['error']
    else:
        error = response.text
    click.echo("PGB Error: {error}".format(error=error), err=True)
    return True


def get_token(directory, token_url):
    token = get_pgb_config(directory, 'token')
    if token:
        return token

    click.echo(
        "Enter your Phonegap Build username and password to request an"
        " authentication token.  The token will be stored in"
        " {directory}/wq-pgb.yml."
        .format(
            directory=directory,
        )
    )
    username = click.prompt("Username")
    password = click.prompt("Password", hide_input=True)
    response = requests.post(token_url, auth=(username, password))

    if check_error(response):
        return

    token = response.json()['token']
    set_pgb_config(directory, 'token', token)
    return token



import os
from wq.core import wq
import click


@wq.command()
@click.option(
    '--filename', '-f', default='version.txt',
    help="Name of text file (default is version.txt)"
)
@click.option('--jsout', help="Name of an AMD module (e.g. myapp/version.js)")
@click.argument('version')
def setversion(**conf):
    """
    Update version.txt (and version.js).  Useful for keeping track of which
    version has been deployed.  The version.js AMD module can be referenced
    within your application to notify users.
    """
    if conf['version'] is None:
        if os.path.exists(conf['filename']):
            version = open(conf['filename']).read().strip()
        else:
            version = ""
    else:
        version = conf['version']
        vtxt = open(conf['filename'], 'w')
        vtxt.write(version)
        vtxt.close()

    if conf['jsout']:
        # Update version.js
        vjs = open(conf['jsout'], 'w')
        vjs.write(VERSIONJS_TMPL % version)
        vjs.close()
        click.echo('%s: %s' % (conf['jsout'], version))
    else:
        click.echo('Application version: %s' % version)

    return version

VERSIONJS_TMPL = """define(function(){return "%s";});"""



from wq.core import wq
import click

from .collect import collectjson
from .setversion import setversion
from .appcache import appcache
from .compilers import optimize, babel, scss, mustache
from .init import init


@wq.command()
@wq.pass_config
@click.pass_context
@click.argument('version')
def build(ctx, config, version):
    """
    Compile and optimize an application.

    \b
    Runs the following in order:
        wq init
        wq setversion
        wq collectjson (if configured)
        wq scss        (if configured)
        wq mustache    (if configured)
        wq optimize
        wq babel       (if configured)
        wq appcache    (if configured)
    """
    if 'optimize' not in config:
        raise click.UsageError(
            "optimize section not found in %s" % config.filename
        )

    def run(command, **kwargs):
        confs = config.get(command.name, {})
        if not isinstance(confs, list):
            confs = [confs]
        for conf in confs:
            conf.update(kwargs)
            ctx.invoke(command, **conf)

    run(init)

    # Save version information
    run(setversion, version=version)

    for command in (collectjson, scss, mustache):
        if command.name in config:
            run(command)

    # Compile Javascript / CSS (using r.js)
    ctx.invoke(optimize)

    # Convert to ES5 via babel.js
    if 'babel' in config:
        ctx.invoke(babel)

    # Generate HTML5 Cache manifests
    if 'appcache' in config:
        ctx.invoke(appcache, version=version)



from wq.core import wq
import click
import os
import re


@wq.command()
@click.argument('version')
@wq.pass_config
def appcache(config, version):
    """
    Generate an HTML5 appcache manifest.  Should be run after wq optimize, as
    some of the manifest entries will be inferred from the build log.

    A manifest will be created for both the source directory and the build
    directory, so you can test offline capabilities even when running off of
    the source AMD files.
    """
    if 'appcache' not in config:
        raise click.UsageError(
            "appcache section not found in %s" % config.filename
        )
    if 'optimize' not in config:
        raise click.UsageError(
            "optimize section not found in %s" % config.filename
        )

    conf = config['appcache']
    indir = config['optimize']['appDir']
    outdir = config['optimize']['dir']

    # Open output files
    s_acpath = indir + '/' + conf['name']
    b_acpath = outdir + '/' + conf['name']
    s_ac = open(s_acpath, 'w')
    b_ac = open(b_acpath, 'w')

    # Start in source directory - read @imports from main CSS file
    s_css = [conf['css']]
    s_css.extend(_parse_css_urls(
        indir, conf['css'], conf.get('css-ignore', None)
    ))

    # Built CSS file contains image URLs from the @import-ed CSS files above
    images = _parse_css_urls(outdir, conf['css'], conf.get('css-ignore', None))

    # build.txt contains a list of built javascript files and their sources
    s_js, b_js = _parse_js_buildfile(outdir + '/build.txt')

    b_css = [conf['css']]

    # Collect path names and create appcaches
    cache = list(conf['cache'])
    source_cache = cache + s_js + s_css + images
    built_cache = cache + b_js + b_css + images

    network = list(conf['network'])
    fallback = list(conf['fallback'])

    s_ac.write(APPCACHE_TMPL % {
        'version':  '%s_dev' % version,
        'cache':    '\n'.join(source_cache),
        'network':  '\n'.join(network),
        'fallback': '\n'.join(fallback)
    })
    b_ac.write(APPCACHE_TMPL % {
        'version':  version,
        'cache':    '\n'.join(built_cache),
        'network':  '\n'.join(network),
        'fallback': '\n'.join(fallback)
    })
    s_ac.close()
    b_ac.close()
    print("%s: %s items" % (s_acpath, len(source_cache + network + fallback)))
    print("%s: %s items" % (b_acpath, len(built_cache + network + fallback)))


def _parse_js_buildfile(filename):
    text = open(filename).read()
    sources = []
    built = []
    for group in text[1:].split('\n\n'):
        files = group.split('\n')

        if files[0][-2:] != 'js':
            continue
        sources.extend(files[2:])
        built.append(files[0])

    return sources, built


def _parse_css_urls(path, filename, ignore=None):
    cur = os.getcwd()
    os.chdir(path)
    text = open(filename).read()
    base = os.path.dirname(filename)
    if ignore:
        ignore_re = re.compile(ignore)

    def parse_url(url):
        if url.startswith('data:') or url.startswith('#'):
            return None
        if url.startswith('http://') or url.startswith('https://'):
            return url
        if ignore and ignore_re.match(url):
            return None

        url = os.path.normpath(base + '/' + url)
        if os.sep != '/':
            url = url.replace(os.sep, '/')
        if ignore and ignore_re.match(url):
            return None
        return url

    urls = []
    for line in text.split('\n'):
        m = re.search(r"url\(['\" ]*(.+?)['\" ]*\)", line)
        if m:
            url = parse_url(m.group(1))
            if url:
                urls.append(url)
    os.chdir(cur)
    return urls

APPCACHE_TMPL = """CACHE MANIFEST

# Version %(version)s

CACHE:
%(cache)s

NETWORK:
%(network)s

FALLBACK:
%(fallback)s
"""



import os
from setuptools import setup

LONG_DESCRIPTION = """
Offline-capable HTML5 web and hybrid apps for citizen science field data collection.
"""

def readme():
    try:
        readme = open('README.md')
    except IOError:
        return LONG_DESCRIPTION
    else:
        return readme.read()


def list_package_data(root):
    """
    List top level js, css, & scss folders for inclusion as package data
    """
    paths = []
    for base, dirs, files in os.walk(root):
        paths.extend([os.path.join(base, name) for name in files])
    return paths


def create_wq_namespace():
    """
    Generate the wq namespace package
    (not checked in, as it technically is the parent of this folder)
>    """
    if os.path.isdir("wq"):
        return
    os.makedirs("wq")
    init = open(os.path.join("wq", "__init__.py"), 'w')
    init.write("__import__('pkg_resources').declare_namespace(__name__)")
    init.close()


create_wq_namespace()
package_data = []
for folder in ['js', 'css', 'scss']:
    package_data.extend(list_package_data(folder))

setup(
    name='wq.app',
    use_scm_version=True,
    author='S. Andrew Sheppard',
    author_email='andrew@wq.io',
    url='https://wq.io/wq.app',
    license='MIT',
    packages=['wq', 'wq.app', 'wq.app.build'],
    package_dir={
        'wq.app': '.',
        'wq.app.build': './build',
    },
    package_data={
        'wq.app': package_data
    },
    install_requires=[
        'wq.core',
        'pyScss>=1.3',
        'compass-stylesheets==0.12.6',
        'PyYAML',
        'requests',
        'pystache',
        'Pillow',
        'requirejs>=0.2.0',
        'PyBabeljs',
    ],
    namespace_packages=['wq'],
    description=LONG_DESCRIPTION.strip(),
    long_description=readme(),
    long_description_content_type='text/markdown',
    entry_points={'wq': 'wq.app=wq.app.build'},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: JavaScript',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Text Processing :: Markup :: HTML',
        'Topic :: Scientific/Engineering :: GIS',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Pre-processors',
    ],
    tests_require=[
        'html-json-forms',
    ],
    setup_requires=[
        'setuptools_scm',
    ],
    project_urls={
        'Homepage': 'https://wq.io/wq.app',
        'Documentation': 'https://wq.io/docs/',
        'Source': 'https://github.com/wq/wq.app',
        'Release Notes': 'https://github.com/wq/wq.app/releases',
        'Issues': 'https://github.com/wq/wq.app/issues',
        'CI': 'https://travis-ci.org/wq/wq.app',
    },
)
